/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * Qt Creator 2.2.1 / Qt 4.7.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <QByteArray>
#include <QThread>
#include "adconverter.h"

extern "C" {
    #include <string.h>
    #include "usbhid.h"
}


extern usbDevice_t *usbdev;
extern int usb_last_err;



class Sleep : public QThread
{
public:
    static void msleep(unsigned long msecs) {
        QThread::msleep(msecs);
    }
};

char *fillbuffer(char *buffer, char command, int data)
{
    //dummy report ID at address 0
    memset(buffer, 0, USB_HID_MSG_SIZE+1);
    buffer[1] = command;               //usbcommand and first byte
    buffer[5] = (char)((data >> 24) & 0xff);
    buffer[6] = (char)((data >> 16) & 0xff);
    buffer[7] = (char)((data >> 8) & 0xff);
    buffer[8] = (char)(data & 0xff);

    return buffer;
}


int getADValue(char command)
{
    int retry, err, len = USB_HID_MSG_SIZE+1;
    char buffer[USB_HID_MSG_SIZE+2];
    buffer[0] = command;

    if (!usbdev) {
        usb_last_err = USBOPEN_ERR_NOTFOUND;
        return -1;
    }

    //write next GET_REPORT ID
    for (retry=1; retry<=USB_HID_REPORT_RETRY; retry++) {
        if ((err = usbhidSetReport(usbdev, fillbuffer(buffer, USB_CMD_WRITE_COMMAND, command), USB_HID_MSG_SIZE+1)) == 0) {
            break;
        } else {
            if (retry == USB_HID_REPORT_RETRY) {
                usb_last_err = err;
                return -1;
            }
            Sleep::msleep(1000);
        }
    }

    for (retry=1; retry<=USB_HID_REPORT_RETRY; retry++) {
        if ((err = usbhidGetReport(usbdev, buffer[0], buffer, &len)) == 0) {
            break;
        } else {
            if (retry == USB_HID_REPORT_RETRY) {
                usb_last_err = err;
                return -1;
            }
            Sleep::msleep(1000);
        }
    }

    return (unsigned char)buffer[5] << 24 | (unsigned char)buffer[6] << 16 | (unsigned char)buffer[7] << 8 | (unsigned char)buffer[8];
}

int setADValue(char command, int value)
{
    int retry, err;
    char buffer[USB_HID_MSG_SIZE+2];

    if (!usbdev) {
        usb_last_err = USBOPEN_ERR_NOTFOUND;
        return -1;
    }

    for (retry=1; retry<=USB_HID_REPORT_RETRY; retry++) {
        if ((err = usbhidSetReport(usbdev, fillbuffer(buffer, command, value), USB_HID_MSG_SIZE+1)) == 0) {
            break;
        } else {
            if (retry == USB_HID_REPORT_RETRY) {
                usb_last_err = err;
                return -1;
            }
            Sleep::msleep(1000);
        }
    }

    return value;
}

int ADConverter::getConvert(int chnum, int uni, int burnout)
{
    int value;
    char command;

    command = USB_CMD_CONVERT_AINP_AINM | (chnum & 7);
    if (uni) command |= USB_CMD_CONVERT_UNI;
    if (burnout) command |= USB_CMD_CONVERT_BURNOUT;

    value = getADValue(command);
    emit convertChanged(value);
    return value;
}

int ADConverter::getOffset()
{
    int value;

    value = getADValue(USB_CMD_OFFSET);
    emit offsetChanged(value);
    return value;
}

int ADConverter::setOffset(int value)
{
    emit offsetChanged(value);
    return setADValue(USB_CMD_OFFSET, value);
}

int ADConverter::setOffsetSystemCalibration()
{
    int value;

    value = getADValue(USB_CMD_OFFSET_SYSTEMCALIBRATION);
    emit offsetChanged(value);
    return value;
}

int ADConverter::setOffsetSelfCalibration()
{
    int value;

    value = getADValue(USB_CMD_OFFSET_SELFCALIBRATION);
    emit offsetChanged(value);
    return value;
}

int ADConverter::getGain()
{
    int value;

    value = getADValue(USB_CMD_GAIN);
    emit gainChanged(value);
    return value;
}

int ADConverter::setGain(int value)
{
    emit gainChanged(value);
    return setADValue(USB_CMD_GAIN, value);
}

int ADConverter::setGainSystemCalibration()
{
    int value;

    value = getADValue(USB_CMD_GAIN_SYSTEMCALIBRATION);
    emit gainChanged(value);
    return value;
}

int ADConverter::setEeprom()
{
    return setADValue(USB_CMD_WRITE_CONFIG, 0);
}

int ADConverter::resetEeprom()
{
    return setADValue(USB_CMD_RESET_CONFIG, 0);
}

int ADConverter::setReboot()
{
    return setADValue(USB_CMD_REBOOT, 1);
}

int ADConverter::getAuto()
{
    int value;

    value = getADValue(USB_CMD_AUTO);
    emit autoChanged(value);
    return value;
}

int ADConverter::setAuto(int value)
{
    emit autoChanged(value);
    return setADValue(USB_CMD_AUTO, value);
}

int ADConverter::getStartupOffset()
{
    int value;

    value = getADValue(USB_CMD_STARTUP_OFFSET);
    emit startupOffsetChanged(value);
    return value;
}

int ADConverter::setStartupOffset(int value)
{
    emit startupOffsetChanged(value);
    return setADValue(USB_CMD_STARTUP_OFFSET, value);
}


int ADConverter::getStatLoops()
{
    int value;

    value = getADValue(USB_CMD_STAT_LOOPS);
    emit statLoopsChanged(value);
    return value;
}

int ADConverter::setStatLoops(int value)
{
    if (value > STAT_LOOPS_MAX) value = STAT_LOOPS_MAX;
    emit statLoopsChanged(value);
    return setADValue(USB_CMD_STAT_LOOPS, value);
}

int ADConverter::getMean()
{
    int value;

    value = getADValue(USB_CMD_READ_MEAN);
    emit meanChanged(value);
    return value;
}

int ADConverter::getMedian()
{
    int value;

    value = getADValue(USB_CMD_READ_MEDIAN);
    emit medianChanged(value);
    return value;
}

float ADConverter::getVariance()
{
    intfloat rv;

    rv.rv_int = getADValue(USB_CMD_READ_VARIANCE);
    emit varianceChanged(rv.rv_float);
    return rv.rv_float;
}

float ADConverter::getStandardDeviation()
{
    intfloat rv;

    rv.rv_int = getADValue(USB_CMD_READ_STD_DEV);
    emit standardDeviationChanged(rv.rv_float);
    return rv.rv_float;
}
