/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * Qt Creator 2.2.1 / Qt 4.7.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/



#ifndef ADCONVERTER_H
#define ADCONVERTER_H

#include    <QObject>

#define USB_HID_REPORT_RETRY                3
#define USB_HID_MSG_SIZE                    8

#define AD_TYPE_AD7796                      0
#define AD_TYPE_AD7797                      1

#define STAT_LOOPS_MAX                      256

#define USB_CMD_WRITE_COMMAND               0x08
#define USB_CMD_STARTUP_OFFSET              0x09
#define USB_CMD_OFFSET                      0x0a
#define USB_CMD_GAIN                        0x0b
#define USB_CMD_OFFSET_SELFCALIBRATION      0x0c
#define USB_CMD_OFFSET_SYSTEMCALIBRATION    0x0d
#define USB_CMD_GAIN_SYSTEMCALIBRATION      0x0e
#define USB_CMD_WRITE_CONFIG                0x0f
#define USB_CMD_RESET_CONFIG                0x18
#define USB_CMD_AUTO                        0x19
#define USB_CMD_STAT_LOOPS                  0x1a
#define USB_CMD_READ_MEAN                   0x1b
#define USB_CMD_READ_VARIANCE               0x1c
#define USB_CMD_READ_STD_DEV                0x1d
#define USB_CMD_READ_MEDIAN                 0x1e
#define USB_CMD_REBOOT                      0x1f

#define	USB_CMD_CONVERT_AINP_AINM           0x20
#define	USB_CMD_CONVERT_AINM_AINM           0x23
#define	USB_CMD_CONVERT_TEMP                0x26
#define	USB_CMD_CONVERT_VDD                 0x27
#define	USB_CMD_CONVERT_UNI                 0x40
#define	USB_CMD_CONVERT_BURNOUT             0x80


class ADConverter  : public QObject
{
    typedef union {
        int rv_int;
        float rv_float;
    } intfloat;

    Q_OBJECT
    public:
        ADConverter() {}
    public slots:
        int getConvert(int chnum, int uni, int burnout);
        int getOffset();
        int setOffset(int value = 0x800000);
        int setOffsetSystemCalibration();
        int setOffsetSelfCalibration();
        int getGain();
        int setGain(int value = 0x550000);
        int setGainSystemCalibration();
        int setEeprom();
        int resetEeprom();
        int getAuto();
        int setAuto(int auto_interval);
        int getStartupOffset();
        int setStartupOffset(int startup_offset);
        int getStatLoops();
        int setStatLoops(int loops);
        int setReboot();
        int getMean();
        int getMedian();
        float getVariance();
        float getStandardDeviation();
    signals:
        void convertChanged(int);
        void offsetChanged(int);
        void gainChanged(int);
        void autoChanged(int);
        void startupOffsetChanged(int);
        void statLoopsChanged(int);
        void meanChanged(int);
        void medianChanged(int);
        void varianceChanged(float);
        void standardDeviationChanged(float);
    private:
        int convert;
        int offset;
        int gain;
        int autoInterval;
        int startupOffset;
        int statLoops;
        int mean;
        int median;
        float variance;
        float standardDeviation;
};



#endif // ADCONVERTER_H
